Package: csmbuilder
Type: Package
Title: A Collection of Tools for Building Cropping System Models
Version: 0.1.0
Authors@R: c(
    person(c("Phillip", "D"), "Alderman",
           email = "phillip.alderman@okstate.edu",
           role = c("aut", "cre")),
    person("Pratishtha", "Poudel",
           role = "aut"))
Description: A collection of tools for designing, implementing,
    testing, documenting and visualizing dynamic simulation
    cropping system models. Models are specified as a combination of
    state variables, parameters, intermediate factors and input data that
    define a system of ordinary differential equations. Specified models
    can be used to simulate dynamic processes using numerical
    integration algorithms.
License: GPL-3
Encoding: UTF-8
RoxygenNote: 7.3.2
Depends: R (>= 4.2.0)
Suggests: deSolve, Rcpp, tinytest
NeedsCompilation: no
Packaged: 2026-03-30 20:32:52 UTC; palderman
Author: Phillip D Alderman [aut, cre],
  Pratishtha Poudel [aut]
Maintainer: Phillip D Alderman <phillip.alderman@okstate.edu>
Repository: CRAN
Date/Publication: 2026-04-03 08:20:15 UTC
Built: R 4.6.0; ; 2026-04-25 18:00:52 UTC; unix
